/*  osdchatd - on-screen display messaging daemon
 *  Copyright (c) 2003 by Mike Smelov  <mike-at-argoncorp-dot-com>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  See LICENSE for details.
 *
 */

 /* common.c */

#include "common.h"

/* global vars */
char *prog_name;		/* program's name -- modified arg0 */

/* sanity checker */
void set_limits(int *integer, int low, int high)
{
	if (*integer < low) {
		printDBG(LVL_ERROR, "%d too low.  Setting to %d.\n", *integer, low);
		*integer = low;
	} else if (*integer > high) {
		printDBG(LVL_ERROR, "%d too high.  Setting to %d.\n", *integer, high);
		*integer = high;
	}
}

/* set prog_name from arg0 */
void set_prog_name(char *arg0)
{
	/* make sure we are not exec'ed with a null arg0 */
	if (*arg0 == NIL)
		printDBG(LVL_FATERR, "Why are you exec'ing me with a null arg0 ?\n");

	if (prog_name != NULL)
		printDBG(LVL_FATERR, "set_prog_name():  Pointer change\n");

	/* set program name */
	if ( (prog_name = strrchr(arg0, '/')) == NULL )
		prog_name = arg0;
	else if (*(prog_name+1) == NIL)
		printDBG(LVL_FATERR, "Why are you exec'ing me with arg0 ending with a slash (/) ?\n");
	else
		prog_name++;
}

/* print debug message */
void printDBG(int level, const char *format, ...)
{
	char buf[BUFSIZE];

	if (level == LVL_DEBUG && debug == 0) {
		return;
	}

	va_list ap;
	va_start(ap, format);
	vsnprintf(buf, sizeof(buf), format, ap);
	va_end(ap);

	switch(level) {
	case LVL_INFO:
	case LVL_DEBUG:
		fprintf(stdout, "%s", buf);
		fflush(stdout);
		break;
	case LVL_FATERR:
	case LVL_ERROR:
		fprintf(stderr, "%s", buf);
		fflush(stderr);
		break;
	case LVL_FATAL:
	case LVL_SYSLOG:
		openlog(prog_name, LOG_PID, LOG_DAEMON);
		syslog(LOG_ERR, "%.500s", buf);
		closelog();

		if (debug == 0)
			break;

		fprintf(stdout, "%s\n", buf);
		fflush(stdout);
		break;
	}

	if (level == LVL_FATAL || level == LVL_FATERR)
		exit(EXIT_FAILURE);
}

/* make sure all reads+writes are complete (blocked io) */
ssize_t complete_io(ssize_t (*func)(), int fd, void *buf, size_t count)
{
	void *b = buf;
	ssize_t nread, p = 0;

	while (count > p) {
		nread = (func) (fd, b + p, count - p);

		if (nread < 0) {
			if (errno == EINTR || errno == EAGAIN || errno == EWOULDBLOCK)
				continue;
		} else if (nread == 0) {
			return nread;
		} else {
			p += nread;
		}
	}

	return p;
}

/* noisier malloc */
void *xmalloc(size_t size)
{
	void *p;

	if (size == 0)
		printDBG(LVL_FATAL, "xmalloc: size zero");

	p = malloc(size);

	if (p == NULL)
		printDBG(LVL_FATAL, "xmalloc: out of memory (allocating %lu bytes)", (u_long) size);

	return p;
}

